@@ -470,9 +470,9 @@ gsppa_send_area (GnomeScanPreviewPluginArea *gsppa)
     gdouble res = gnome_scan_preview_area_get_resolution (GNOME_SCAN_PREVIEW_AREA (gspp->preview_area));
     
     /* origin */
-    GdkPoint *o = gnome_scan_settings_get_pointer (s, "origin");
-    o->x = r.x;
-    o->y = r.y;
+    GSPoint *o = gnome_scan_settings_get_pointer (s, "origin");
+    o->x = gs_convert_to_mm (r.x, GS_UNIT_PIXEL, res);
+    o->y = gs_convert_to_mm(r.y, GS_UNIT_PIXEL, res);
     
     /* paper-size (if manual) */
     GtkPaperSize *p = gnome_scan_settings_get_boxed (s, "paper-size",
@@ -483,12 +483,17 @@ gsppa_send_area (GnomeScanPreviewPluginArea *gsppa)
                  gs_convert (r.height, GS_UNIT_PIXEL, GS_UNIT_MM, res));
         g_boxed_free (GTK_TYPE_PAPER_SIZE, p);
         p = gtk_paper_size_new_custom ("manual", _("Manual"),
-                                       gs_convert (r.width, GS_UNIT_PIXEL, GS_UNIT_MM, res),
-                                       gs_convert (r.height, GS_UNIT_PIXEL, GS_UNIT_MM, res),
+                                       gs_convert (MIN(r.width, r.height), GS_UNIT_PIXEL, GS_UNIT_MM, res),
+                                       gs_convert (MAX(r.width, r.height), GS_UNIT_PIXEL, GS_UNIT_MM, res),
                                        GTK_UNIT_MM);
+		/* update orientation according to area extent */
+		GtkPageOrientation or = r.width > r.height ? GTK_PAGE_ORIENTATION_LANDSCAPE : GTK_PAGE_ORIENTATION_PORTRAIT;
+		gnome_scan_settings_set_enum (s, "page-orientation",
+									  GTK_TYPE_PAGE_ORIENTATION, or);
     }
     else if (g_str_equal (gtk_paper_size_get_name (p), "maximal")) {
-        o->x = o->y = 0;
+		/* set origin to (0,0) for maximal paper-size */
+        o->x = o->y = 0.;
     }
     
     gnome_scan_settings_set_pointer (s, "origin", o);
@@ -506,7 +511,7 @@ gsppa_send_area (GnomeScanPreviewPluginArea *gsppa)
 static void
 gsppa_set_roi (GnomeScanPreviewPlugin *gspp)
 {
-    GdkPoint *p;
+    GSPoint *p;
     GtkPaperSize *ps;
     GtkPageOrientation o;
     gint tmp;
@@ -520,18 +525,18 @@ gsppa_set_roi (GnomeScanPreviewPlugin *gspp)
     o = gnome_scan_settings_get_enum(gspp->settings, "page-orientation",
                                      GTK_TYPE_PAGE_ORIENTATION);
     if (!p) {
-        p = g_new0 (GdkPoint, 1);
+        p = g_new0 (GSPoint, 1);
         gnome_scan_settings_set_pointer (gspp->settings, "origin", p);
     }
     
-    r.x = p->x;
-    r.y = p->y;
+    r.x = gs_convert_from_mm (p->x, GS_UNIT_PIXEL, res);
+    r.y = gs_convert_from_mm (p->y, GS_UNIT_PIXEL, res);
     
     ps = gnome_scan_settings_get_boxed (gspp->settings, "paper-size",
                                         GTK_TYPE_PAPER_SIZE);
     
-    r.width = (gint) gs_convert (gtk_paper_size_get_width (ps, GTK_UNIT_MM), GTK_UNIT_MM, GS_UNIT_PIXEL, res);
-    r.height = (gint) gs_convert (gtk_paper_size_get_height (ps, GTK_UNIT_MM), GTK_UNIT_MM, GS_UNIT_PIXEL, res);
+    r.width = (gint) gs_convert_from_mm (gtk_paper_size_get_width (ps, GTK_UNIT_MM), GS_UNIT_PIXEL, res);
+    r.height = (gint) gs_convert_from_mm (gtk_paper_size_get_height (ps, GTK_UNIT_MM), GS_UNIT_PIXEL, res);
     
     switch(o) {
         case GTK_PAGE_ORIENTATION_LANDSCAPE:
